@@ -21,11 +21,12 @@
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
+#include <glib/gi18n.h>
 
-#include "currency.h"
 #include "unittest.h"
 #include "math-window.h"
 #include "mp-equation.h"
+#include "unit-manager.h"
 
 static GSettings *settings = NULL;
 
@@ -39,6 +40,13 @@ version(const gchar *progname)
 }
 
 
+static int
+do_convert(const MPNumber *x, const char *x_units, const char *z_units, MPNumber *z, void *data)
+{
+    return unit_manager_convert(unit_manager_get_default(), x, x_units, z_units, z);
+}
+
+
 static void
 solve(const char *equation)
 {
@@ -51,6 +59,7 @@ solve(const char *equation)
     options.base = 10;
     options.wordlen = 32;
     options.angle_units = MP_DEGREES;
+    options.convert = do_convert;
 
     error = mp_equation_parse(equation, &options, &result, NULL);
     if(error == PARSER_ERR_MP) {
@@ -198,7 +207,6 @@ quit_cb(MathWindow *window)
     g_settings_set_string(settings, "target-units", math_equation_get_target_units(equation));
     g_settings_sync();
 
-    currency_free_resources();
     gtk_main_quit();
 }
 
