@@ -32,13 +32,13 @@ typedef struct MathDisplayPrivate MathDisplayPrivate;
 
 typedef struct
 {
-    GtkVBox parent_instance;
+    GtkViewport parent_instance;
     MathDisplayPrivate *priv;
 } MathDisplay;
 
 typedef struct
 {
-    GtkVBoxClass parent_class;
+    GtkViewportClass parent_class;
 } MathDisplayClass;
 
 GType math_display_get_type(void);
@@ -49,4 +49,6 @@ MathDisplay *math_display_new_with_equation(MathEquation *equation);
 
 MathEquation *math_display_get_equation(MathDisplay *display);
 
+G_END_DECLS
+
 #endif /* MATH_DISPLAY_H */
