@@ -24,14 +24,15 @@
 #include <math.h>
 #include <errno.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include "math-equation.h"
 
 #include "mp.h"
 #include "mp-equation.h"
 #include "mp-serializer.h"
-#include "currency.h"
 #include "math-enums.h"
+#include "unit-manager.h"
 
 
 enum {
@@ -266,7 +267,7 @@ reformat_separators(MathEquation *equation)
         }
         else if (c == mp_serializer_get_thousands_separator(equation->priv->serializer)) {
             /* Didn't expect thousands separator - delete it */
-            if (!expect_tsep) {
+            if (!expect_tsep && in_number) {
                 GtkTextIter start, end;
                 gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(equation), &start, offset);
                 gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(equation), &end, offset + 1);
@@ -640,10 +641,6 @@ math_equation_get_angle_units(MathEquation *equation)
 void
 math_equation_set_source_currency(MathEquation *equation, const gchar *currency)
 {
-    // FIXME: Pick based on locale  
-    if (!currency || currency[0] == '\0')
-        currency = currency_info[0].short_name;
-
     if (strcmp(equation->priv->source_currency, currency) == 0)
         return;
     g_free(equation->priv->source_currency);
@@ -662,10 +659,6 @@ math_equation_get_source_currency(MathEquation *equation)
 void
 math_equation_set_target_currency(MathEquation *equation, const gchar *currency)
 {
-    // FIXME: Pick based on locale  
-    if (!currency || currency[0] == '\0')
-        currency = currency_info[0].short_name;
-
     if (strcmp(equation->priv->target_currency, currency) == 0)
         return;
     g_free(equation->priv->target_currency);
@@ -881,7 +874,7 @@ math_equation_store(MathEquation *equation, const gchar *name)
     if (!math_equation_get_number(equation, &t))
         math_equation_set_status(equation, _("No sane value to store"));
     else
-        math_variables_set_value(equation->priv->variables, name, &t);
+        math_variables_set(equation->priv->variables, name, &t);
 }
 
 
@@ -1033,7 +1026,7 @@ variable_is_defined(const char *name, void *data)
     }
     g_free(lower_name);
 
-    return math_variables_get_value(equation->priv->variables, name) != NULL;
+    return math_variables_get(equation->priv->variables, name) != NULL;
 }
 
 
@@ -1054,7 +1047,7 @@ get_variable(const char *name, MPNumber *z, void *data)
     else if (strcmp(lower_name, "ans") == 0)
         mp_set_from_mp(&equation->priv->state.ans, z);
     else {
-        t = math_variables_get_value(equation->priv->variables, name);
+        t = math_variables_get(equation->priv->variables, name);
         if (t)
             mp_set_from_mp(t, z);
         else
@@ -1072,14 +1065,14 @@ set_variable(const char *name, const MPNumber *x, void *data)
 {
     MathEquation *equation = data;
     /* FIXME: Don't allow writing to built-in variables, e.g. ans, rand, sin, ... */
-    math_variables_set_value(equation->priv->variables, name, x);
+    math_variables_set(equation->priv->variables, name, x);
 }
 
 
 static int
 convert(const MPNumber *x, const char *x_units, const char *z_units, MPNumber *z, void *data)
-{   
-    return currency_convert(x, x_units, z_units, z);
+{
+    return unit_manager_convert(unit_manager_get_default(), x, x_units, z_units, z);
 }
 
 
@@ -1849,8 +1842,8 @@ math_equation_init(MathEquation *equation)
     equation->priv->word_size = 32;
     equation->priv->angle_units = MP_DEGREES;
     // FIXME: Pick based on locale
-    equation->priv->source_currency = g_strdup(currency_info[0].short_name);
-    equation->priv->target_currency = g_strdup(currency_info[0].short_name);
+    equation->priv->source_currency = g_strdup("");
+    equation->priv->target_currency = g_strdup("");
     equation->priv->source_units = g_strdup("");
     equation->priv->target_units = g_strdup("");
     equation->priv->serializer = mp_serializer_new(MP_DISPLAY_FORMAT_AUTOMATIC, 10, 9);
